"use strict";
const vueDemi = require("vue-demi");
const Fetch = require("./Fetch");
const _interopDefaultLegacy = (e) => e && typeof e === "object" && "default" in e ? e : { default: e };
const Fetch__default = /* @__PURE__ */ _interopDefaultLegacy(Fetch);
function useRequestImplement(service, options = {}, plugins = []) {
  const { initialData = void 0, manual = false, ready = true, ...rest } = options;
  const fetchOptions = {
    manual,
    ready,
    ...rest
  };
  const serviceRef = vueDemi.ref(service);
  const state = vueDemi.reactive({
    data: initialData,
    loading: false,
    params: void 0,
    error: void 0
  });
  const setState = (s, field) => {
    if (field) {
      state[field] = s;
    } else {
      state.data = s.data;
      state.loading = s.loading;
      state.error = s.error;
      state.params = s.params;
    }
  };
  const initState = plugins.map((p) => {
    var _a;
    return (_a = p == null ? void 0 : p.onInit) == null ? void 0 : _a.call(p, fetchOptions);
  }).filter(Boolean);
  const fetchInstance = new Fetch__default.default(
    serviceRef,
    fetchOptions,
    setState,
    Object.assign({}, ...initState, state)
  );
  fetchInstance.options = fetchOptions;
  fetchInstance.pluginImpls = plugins.map((p) => {
    return p(fetchInstance, fetchOptions);
  });
  vueDemi.onMounted(() => {
    if (!manual) {
      const params = fetchInstance.state.params || options.defaultParams || [];
      if (vueDemi.unref(ready))
        fetchInstance.run(...params);
    }
  });
  vueDemi.onUnmounted(() => {
    fetchInstance.cancel();
  });
  return {
    ...vueDemi.toRefs(state),
    cancel: fetchInstance.cancel.bind(fetchInstance),
    refresh: fetchInstance.refresh.bind(fetchInstance),
    refreshAsync: fetchInstance.refreshAsync.bind(fetchInstance),
    run: fetchInstance.run.bind(fetchInstance),
    runAsync: fetchInstance.runAsync.bind(fetchInstance),
    mutate: fetchInstance.mutate.bind(fetchInstance)
  };
}
module.exports = useRequestImplement;
